#include <Ethernet.h>
#include <EthernetUdp.h>
#include <Servo.h>

byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };
IPAddress ip(192, 168, 0, 175);

unsigned int localPort = 55278;

char packetBuffer[UDP_TX_PACKET_MAX_SIZE];

EthernetUDP Udp;
Servo myServo;

void setup()
{
  Serial.begin(9600);
  while (!Serial);

  Ethernet.begin(mac, ip);

  if (Ethernet.hardwareStatus() == EthernetNoHardware)
  {
    Serial.println("Ethernet shield was not found.");
    while (true)
      delay(1);
  }

  if (Ethernet.linkStatus() == LinkOFF)
    Serial.println("Ethernet cable is not connected.");

  Udp.begin(localPort);

  myServo.attach(9);
  myServo.write(0);
  
  delay(500);

  Serial.println("Setup complete.");
}

void loop()
{
  int packetSize = Udp.parsePacket();

  if (packetSize)
  {
    Serial.print("Received packet of size ");
    Serial.println(packetSize);

    Serial.print("From ");
    IPAddress remote = Udp.remoteIP();
    for (int i=0; i < 4; i++)
    {
      Serial.print(remote[i], DEC);
      if (i < 3)
        Serial.print(".");
    }
    Serial.print(", port ");
    Serial.println(Udp.remotePort());

    char message[packetSize];
    int fragmentSize = 0;
    int messageIndexer = 0;
    do
    {
      fragmentSize = Udp.read(packetBuffer, UDP_TX_PACKET_MAX_SIZE);
      for (int i = 0; i < fragmentSize + 1; i++)
        message[i + messageIndexer] = packetBuffer[i];

      messageIndexer += fragmentSize;
    }
    while (fragmentSize > 0);

    //Serial.println("Contents of message:");
    //Serial.println(message);

    char flightValueName[] = "airspeed=";
    int searchIndex = 0;
    int searchedStringLength = sizeof(flightValueName) / sizeof(char) - 1;
    int valueLength = 0;
    char valueString[16];
    float airSpeed = 0;
    for (int i = 0; i < packetSize; i++)
    {
      if (message[i] == flightValueName[i - searchIndex] && (i - searchIndex) == (searchedStringLength - 1))
      {
        while (isDigit(message[++i]) || (message[i] == '.'))
          valueString[valueLength++] = message[i];

        airSpeed = atof(valueString);
        break;
      }

      if (message[i] != flightValueName[i - searchIndex])
        i = searchIndex++;
    }

    Serial.print("Air speed: ");
    Serial.println(airSpeed, 6);

    Serial.println();

    float airSpeedKPH = airSpeed * 3.6;
    float servoAngle = airSpeedKPH * 180.0 / 400.0;
    if (servoAngle >= 180)
      servoAngle = 180;
    else if (servoAngle <= 0)
      servoAngle = 0;

    myServo.write(servoAngle);

    Serial.println(servoAngle);
    Serial.println(myServo.read());
  }

  delay(50);
}
